@@ -8,7 +8,7 @@ package Pgreet::Config;
 # A Perl CGI-based web card application for LINUX and probably any
 # other UNIX system supporting standard Perl extensions.
 #
-#   Edouard Lagache, elagache@canebas.org, Copyright (C)  2003, 2004
+#   Edouard Lagache, elagache@canebas.org, Copyright (C)  2003-2005
 #
 # Penguin Greetings (pgreet) consists of a Perl CGI script that
 # handles interactions with users wishing to create and/or
@@ -38,9 +38,9 @@ package Pgreet::Config;
 # It provides for systematic updating of configuration information,
 # interrupt handling, and so on.
 ######################################################################
-# $Id: Config.pm,v 1.31 2004/03/29 21:25:23 elagache Exp $
+# $Id: Config.pm,v 1.34 2005/05/07 16:17:07 elagache Exp $
 
-$VERSION = "0.9.8"; # update after release
+$VERSION = "0.9.9"; # update after release
 
 # Module exporter declarations
 @ISA       = qw(Exporter);
@@ -413,6 +413,11 @@ sub _merge_configs {
   # Create a copy of default hash.
   %{$new_hash} = %{$default_hash};
 
+  # Delete any Embperl_Object setting - this cannot be inherited.
+  if (exists($new_hash->{Embperl_Object})) {
+	delete($new_hash->{Embperl_Object});
+  }
+
   # Change values to reflect changes from secondary site.
   foreach my $value (keys(%{$config_hash})) {
 	if (exists($set_primary_only{$value})) {
@@ -851,7 +856,7 @@ of Penguin Greetings.  they are listed here for completeness:
 
 =head1 COPYRIGHT
 
-Copyright (c) 2003, 2004 Edouard Lagache
+Copyright (c) 2003-2005 Edouard Lagache
 
 This software is released under the GNU General Public License, Version 2.
 For more information, see the COPYING file included with this software or
@@ -867,7 +872,7 @@ Edouard Lagache <pgreetdev@canebas.org>
 
 =head1 VERSION
 
-0.9.8
+0.9.9
 
 =head1 SEE ALSO
 
